// RW Bumpmaster
// ==============
// Imageboard Retrieval & Archiving Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/curator/bumpmaster/bumpmaster.hpp
// -Declarations of the baseline RW Bumpmaster sub-library

#pragma once

#include "board.hpp"
#include "catalog.hpp"
#include "page.hpp"
#include "post.hpp"
#include "site.hpp"
#include "thread.hpp"

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
/** A simple data struct to help with thread's post's information
 */
struct Postdata {    // Lynxchan's JSON tag names:
  string   date;     // creation
  string   message;  // message
  unsigned id;       // postId
};

//------------------------------------------------------------------------------
/** Assign to a post's data struct from a JSON object
 * @param[in] j
 * @param[out] d
 */
void from_json(Json const& j, Postdata& p);

//------------------------------------------------------------------------------
/** Assign to a JSON object from a post's data struct
 * @param[out] j
 * @param[in] d
 */
void to_json(Json& j, Postdata const& p);

//------------------------------------------------------------------------------
/**
 */
class Bumpmaster_b : public RW {
 public:
  Bumpmaster_b();                                    // default ctor
  explicit Bumpmaster_b(std::string const& handle);  // tag-parm'd ctor

  /** Extract site name, board name, from IB catalog URI @a cat_uri
   * @param cat_uri
   * @return
   */
  auto parse_cat_uri(std::string const& cat_uri)
      -> std::tuple<std::string, std::string>;

  /**
   * @param site_nm
   * @param board_nm
   */
  // void chk_board_dir(std::string const& site_nm, std::string const&
  // board_nm);

  /** Assemble an IB catalog JSON URI from key parts
   * @param host
   * @param path
   * @return the assembled URI
   * @note: example resulting form: https://alogs.space/robowaifu/catalog.json
   */
  auto bld_cat_uri(std::string const& host, std::string const& path)
      -> std::string;

  /** Assemble an IB thread JSON URI from key parts
   * @param host
   * @param path
   * @param thrd_id
   * @return the assembled URI
   * @note: example resulting form: https://alogs.space/robowaifu/res/14409.json
   */
  auto bld_thrd_uri(std::string const& host,
                    std::string const& path,
                    unsigned const     thrd_id) -> std::string;

  /**
   * @param thread
   * @return
   */
  auto rd_thread_id(Json const& thread) -> unsigned;

  /**
   * @param cat_json
   * @param site
   * @param board
   * @return
   */
  auto bump_thrd_uris(Json const&        cat_json,
                      std::string const& site,
                      std::string const& board) -> std::vector<std::string>;

  /**
   * @param thrd_json
   * @return
   */
  auto thrd_posts(Json const& thrd_json) -> std::vector<Postdata>;
};

//------------------------------------------------------------------------------
/**
 */
class Bumpmaster : public Bumpmaster_b {
 public:
  Bumpmaster();                                    // default ctor
  explicit Bumpmaster(std::string const& handle);  // tag-parm'd ctor

  inline auto board() const -> Board& { return board_; }
  inline auto catalog() const -> Catalog& { return catalog_; }
  inline auto page() const -> Page& { return page_; }
  inline auto post() const -> Post& { return post_; }
  inline auto site() const -> Site& { return site_; }
  inline auto thread() const -> Thread& { return thread_; }

  inline auto waifusearch() const -> Waifusearch& { return waifusearch_; }

 private:
  /**
   */
  void preflight();

  unique_ptr<Board_b>   _board_{make_unique<Board_b>(t(), "bumpmaster_board")};
  unique_ptr<Catalog_b> _catalog_{
      make_unique<Catalog_b>(t(), "bumpmaster_catalog")};
  unique_ptr<Page_b>   _page_{make_unique<Page_b>(t(), "bumpmaster_page")};
  unique_ptr<Post_b>   _post_{make_unique<Post_b>(t(), "bumpmaster_post")};
  unique_ptr<Site_b>   _site_{make_unique<Site_b>(t(), "bumpmaster_site")};
  unique_ptr<Thread_b> _thread_{
      make_unique<Thread_b>(t(), "bumpmaster_thread")};

  unique_ptr<Waifusearch_b> _waifusearch_{
      make_unique<Waifusearch_b>("bumpmaster_waifusearch")};

  Board&   board_{reference_to<Board>(_board_.get())};
  Catalog& catalog_{reference_to<Catalog>(_catalog_.get())};
  Page&    page_{reference_to<Page>(_page_.get())};
  Post&    post_{reference_to<Post>(_post_.get())};
  Site&    site_{reference_to<Site>(_site_.get())};
  Thread&  thread_{reference_to<Thread>(_thread_.get())};

  Waifusearch& waifusearch_{reference_to<Waifusearch>(_waifusearch_.get())};

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

 public:
  /** Pull a JSON dolldrop
   * @param json_uri
   * @return @c Json object constructed from the dolldrop
   */
  auto pull_json_drop(std::string const& json_uri) -> Json;

  inline auto dolldrop() const -> Dolldrop& { return dolldrop_; }

 private:
  unique_ptr<Dolldrop_b> _dolldrop_{
      make_unique<Dolldrop_b>(t(), "bumpmaster_dolldrop")};

  Dolldrop& dolldrop_{reference_to<Dolldrop>(_dolldrop_.get())};

#endif  // __has_include(<curl_easy.h>)
};

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v220219
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
